# Generated by Django 2.2.14 on 2020-08-28 14:34

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Lock',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('lock_name', models.CharField(max_length=128, unique=True, verbose_name='Lock Name')),
                ('worker_name', models.CharField(max_length=128, verbose_name='Worker Name')),
                ('lock_time', models.DateTimeField(verbose_name='Lock Time')),
                ('expire_time', models.DateTimeField(verbose_name='Expired Time')),
            ],
            options={
                'verbose_name': 'Lock',
                'verbose_name_plural': 'Locks',
            },
        ),
    ]
