from django.conf import settings

DJANGO_DB_LOCK_AUTO_REGISTER_MODEL = getattr(settings, "DJANGO_DB_LOCK_AUTO_REGISTER_MODEL", True)
DJANGO_DB_LOCK_AUTO_REGISTER_ADMIN = getattr(settings, "DJANGO_DB_LOCK_AUTO_REGISTER_ADMIN", True)
DJANGO_DB_LOCK_AUTO_REGISTER_SERVICES = getattr(settings, "DJANGO_DB_LOCK_AUTO_REGISTER_SERVICES", True)
DJANGO_DB_LOCK_APP_LABEL = getattr(settings, "DJANGO_DB_LOCK_APP_LABEL", "django_db_lock")
DJANGO_DB_LOCK_API_SERVER = getattr(settings, "DJANGO_DB_LOCK_API_SERVER", None)
DJANGO_DB_LOCK_ACQUIRE_LOCK_PATH = getattr(settings, "DJANGO_DB_LOCK_ACQUIRE_LOCK_PATH", "acquireLock")
DJANGO_DB_LOCK_RELEASE_LOCK_PATH = getattr(settings, "DJANGO_DB_LOCK_RELEASE_LOCK_PATH", "releaseLock")

DJANGO_DB_LOCK_REDIS_CACHE_NAME = getattr(settings, "DJANGO_DB_LOCK_REDIS_CACHE_NAME", "default")
DJANGO_DB_LOCK_USA_REDIS_CACHE = getattr(settings, "DJANGO_DB_LOCK_USA_REDIS_CACHE", True)
DJANGO_DB_LOCK_REDIS_CACHE_PREFIX = getattr(settings, "DJANGO_DB_LOCK_REDIS_CACHE_PREFIX", "django-db-lock:locks")
